﻿Imports ZedGraph

Public Class PoEEndMidspanCollision
    Private randGenerator As System.Random

    Private Sub PoEEndMidspanCollision_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        initializeZedGraph()

        ' Single random generator must be shared by everyone or 
        ' the PSEs "random" numbers (seeded based on CPU tick) will sync up
        randGenerator = New System.Random()
    End Sub

    Private Sub initializeZedGraph()
        ' Inits XY point data structures and colors for graphing panes

        Dim myPane As GraphPane = zGraphA.GraphPane

        '' Alt A Graph
        myPane.Title.Text = "Alt A Endspan Detection"
        myPane.XAxis.Title.Text = "Time, Seconds"
        myPane.YAxis.Title.Text = "Voltage"

        Dim list As New PointPairList()
        ' Initially, a curve is added with no data points (list is empty)
        ' Color is XYZ, and there will be no symbols
        Dim aCurve As LineItem = myPane.AddCurve("Alt A", list, Color.Red, SymbolType.None)
        aCurve.Line.Width = 3.0F
        aCurve.Line.StepType = StepType.ForwardStep

        ' Configure X scale
        myPane.XAxis.MajorGrid.IsVisible = True

        ' Make the Y axis scale red
        myPane.YAxis.Scale.FontSpec.FontColor = Color.Red
        myPane.YAxis.Title.FontSpec.FontColor = Color.Red
        ' Align the Y axis labels so they are flush to the axis
        myPane.YAxis.Scale.Align = AlignP.Inside

        myPane.YAxis.Scale.Max = 1
        myPane.YAxis.Scale.Min = -51

        ' Scale the axes (recompute various graph auto attributes
        zGraphA.AxisChange()
        ' Force redraw (is not guaranteed) (may need to form form too)
        zGraphA.Invalidate()

        '' Alt B Graph
        myPane = zGraphB.GraphPane

        myPane.Title.Text = "Alt B Midspan Detection"
        myPane.XAxis.Title.Text = "Time, Seconds"
        myPane.YAxis.Title.Text = "Voltage"

        list = New PointPairList()
        aCurve = myPane.AddCurve("Alt B", list, Color.Blue, SymbolType.None)
        aCurve.Line.Width = 3.0F
        aCurve.Line.StepType = StepType.ForwardStep

        ' Configure X scale
        myPane.XAxis.MajorGrid.IsVisible = True

        ' Make the Y axis scale red
        myPane.YAxis.Scale.FontSpec.FontColor = Color.Red
        myPane.YAxis.Title.FontSpec.FontColor = Color.Red
        ' Align the Y axis labels so they are flush to the axis
        myPane.YAxis.Scale.Align = AlignP.Inside

        myPane.YAxis.Scale.Max = 1
        myPane.YAxis.Scale.Min = -51

        ' Scale the axes (recompute various graph auto attributes
        zGraphA.AxisChange()
        zGraphB.AxisChange()
        ' Force redraw (is not guaranteed) (may need to form form too)
        zGraphA.Invalidate()
        zGraphB.Invalidate()
    End Sub

    Private Sub bRunSingle_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bRunSingle.Click
        generateScenario()
    End Sub

    Private Sub bStartLoop_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bStartLoop.Click
        ' Do not change the way single/start/stop button and generateScenario interact 
        ' Designed to be compatible with the VB.Net cooperative multi-tasking architecture

        ' Make sure two "theads" don't call generateScenario
        bRunSingle.Enabled = False
        bStartLoop.Enabled = False
        bStopLoop.Enabled = True

        ' Run a series of scenarios until stop button is clicked
        While bStartLoop.Enabled = False
            generateScenario()
            Application.DoEvents() ' Allow GUI to update and handle clicks

            ' Insert delay to make scenarios more visible to naked eye
            Select Case tbSpeed.Value
                Case 1
                    Wait(1000)
                Case 2
                    Wait(250)
                Case 3
                    Wait(50)
                Case Else
                    ' No wait
            End Select

        End While

    End Sub

    Private Sub bStopLoop_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bStopLoop.Click
        ' Make sure two "theads" don't call generateScenario
        bRunSingle.Enabled = True
        bStartLoop.Enabled = True
        bStopLoop.Enabled = False
    End Sub

    Private Sub generateScenario()
        ' Create PSEs based on configuration
        Dim pseA As PSE = New PSE(PSE.pseType.endspan, udEndspanBackoff.Value, _
                                  udTpon.Value, udTdetect.Value, _
                                  randGenerator)
        Dim pseB As PSE = New PSE(PSE.pseType.midspan, udMidspanBackoff.Value, _
                                  udTpon.Value, udTdetect.Value, _
                                  randGenerator)

        Dim scenarioDone As Boolean = False
        Dim ptTime As Double = 0
        Dim detectCollision As Integer = 0

        ' Clear graph points
        Dim listA As IPointListEdit = zGraphA.GraphPane.CurveList(0).Points
        listA.Clear()
        Dim listB As IPointListEdit = zGraphB.GraphPane.CurveList(0).Points
        listB.Clear()

        ' Increant scenario counter
        tbPlugInCount.Text += 1

        ' Advance 1ms at a time
        While Not scenarioDone
            Dim voltA As Double = pseA.tick()
            Dim voltB As Double = pseB.tick()

            '' Single signatures detect collisions
            If rbSingleSig.Checked Then
                If pseA.getState = PSE.pseState.detect And pseB.getState = PSE.pseState.detect Then
                    detectCollision += 1
                Else
                    detectCollision = 0
                End If
                If detectCollision > 5 Then
                    pseA.restart(udEndspanBackoff.Value) ' Could use a different backoff than during new (initial) call
                    pseB.restart(udMidspanBackoff.Value) ' Could use a different backoff than during new (initial) call
                End If
            End If

            '' Enforce rule that PDs must present invalid sig on unpowered pairs
            ' Alt A Powered
            If ((pseA.getState = PSE.pseState.powerUp Or pseA.getState = PSE.pseState.powerOn) And _
                pseB.getState = PSE.pseState.detect) Then
                pseB.restart(udMidspanBackoff.Value) ' Could use a different backoff than during new (initial) call
            End If

            ' Alt B Powered
            If ((pseB.getState = PSE.pseState.powerUp Or pseB.getState = PSE.pseState.powerOn) And _
                pseA.getState = PSE.pseState.detect) Then
                pseA.restart(udMidspanBackoff.Value) ' Could use a different backoff than during new (initial) call
            End If

            '' See if sim is "done"
            If pseA.getState = PSE.pseState.powerOn Or pseB.getState = PSE.pseState.powerOn Then
                ' Sim complete, evaluate result
                If pseA.getState = PSE.pseState.powerOn Then
                    If (pseB.getState = PSE.pseState.tpon) Or _
                       (pseB.getState = PSE.pseState.powerUp) Or _
                       (pseB.getState = PSE.pseState.powerOn) Then
                        tbContentionCount.Text += 1
                        tbResult.Text = "4P Power"
                        tbResult.ForeColor = Color.Red
                    Else
                        tbResult.Text = "2P Power"
                        tbResult.ForeColor = Color.Black
                    End If
                    scenarioDone = True
                ElseIf pseB.getState = PSE.pseState.powerOn Then
                    If (pseA.getState = PSE.pseState.tpon) Or _
                       (pseA.getState = PSE.pseState.powerUp) Or _
                       (pseA.getState = PSE.pseState.powerOn) Then
                        tbContentionCount.Text += 1
                        tbResult.Text = "4P Power"
                        tbResult.ForeColor = Color.Red
                    Else
                        tbResult.Text = "2P Power"
                        tbResult.ForeColor = Color.Black
                    End If
                    scenarioDone = True
                End If
            End If

            ' Abort if timeout
            If ptTime > 100 Then
                tbResult.Text = "Deadlock...timed out at 100 seconds"
                scenarioDone = True
            End If

            ' Graph new point
            listA.Add(ptTime, voltA)
            listB.Add(ptTime, voltB)
            ptTime += 0.001
        End While

        ' Update stats
        tbContentionPercent.Text = Math.Round(100 * tbContentionCount.Text / tbPlugInCount.Text, 1)

        ' Scale the graph axes (recompute various graph auto attributes)
        zGraphA.AxisChange()
        zGraphB.AxisChange()
        ' Request redraw (at next doEvents)
        zGraphA.Invalidate()
        zGraphB.Invalidate()
    End Sub

    Private Sub bClearAll_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bClearAll.Click
        tbPlugInCount.Text = "0"
        tbContentionCount.Text = "0"
        tbContentionPercent.Text = "0"
    End Sub

    Public Sub Wait(ByVal mS As Integer, Optional ByVal CheckEvents As Boolean = False)
        Dim Start As Long = Now.Ticks

        Do
            If CheckEvents Then Application.DoEvents()
        Loop Until Now.Ticks > Start + (mS * TimeSpan.TicksPerMillisecond)

    End Sub
End Class
